#
# bts.rb - ruby interface for debian bts
# Copyright (c) 2002 Masato Taruishi <taru@debian.org>
# Copyright (c) 2006-2007 Junichi Uekawa <dancer@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Currently, this interface has only acquires to create bugs.

require 'debian/bug'
require 'net/http'
require 'uri.rb'
require 'find'
require 'ftools'
require 'zlib'
require 'thread'

module Debian
  module BTS
    class Parser

      def initialize(host, port)
        @host = host
        @port = port
      end

      # use SOAP interface to obtain the index.
      class SoapIndex < Parser
	def initialize(host, port)
          @host = host
          @port = port
	  @indexes = {}
	  @buf = nil
	end

        ParseStep = 200

	def parse(pkgs, severities = ["critical", "grave"])
          require 'debian/btssoap'
          soap = Debian::BTSSOAP::Soap.new(@host, @port)
          sa = Debian::BTSSOAP::StringArray.new
          bugs = Debian::Bugs.new

          reqbugs = soap.get_bugs('severity', severities, 'package', pkgs)

          # the total number of bugs
          max = reqbugs.length
          # progressed amount
          offset_i = 0 

          # process each bug number and get contents, in bach of ParseStep
          reqbugs.each { |bug|
            sa << bug
            if sa.length > ParseStep
              soap.get_status(sa).each { |b| bugs << b }
              offset_i += sa.length
              sa = Debian::BTSSOAP::StringArray.new
              yield "#{((offset_i).to_f*100/max.to_f).to_i}%"
            end
          }
          soap.get_status(sa).each { |b| bugs << b }
          bugs
	end
      end
    end      
  end
end
