# btssoap.rb - ruby interface for Debian BTS SOAP engine
# Copyright (c) 2006-2007 Junichi Uekawa
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require 'soap/rpc/driver'
require 'debian/bug'

module Debian
  module BTSSOAP
    class StringArray < Array; end 
    class Soap
      def initialize(host = "bugs.debian.org", port = 80)
        @server="http://#{host}:#{port}/cgi-bin/soap.cgi"
        @ns = 'Debbugs/SOAP/'
        @drv = SOAP::RPC::Driver.new(@server, @ns)
        @drv.wiredump_dev = STDOUT if $DEBUG
        @drv.add_method('get_status','bugnumber')

        # the number of options should be variable, but I don't know how it should be done
        @drv.add_method('get_bugs','key','value', 'key2', 'value2')
        @drv.options["protocol.http.connect_timeout"] = 999
        @drv.options["protocol.http.send_timeout"] = 999
        @drv.options["protocol.http.receive_timeout"] = 999
        
        # define StringArray
        @map = SOAP::Mapping::Registry.new
        @map.set(StringArray, SOAP::SOAPArray,
                SOAP::Mapping::Registry::TypedArrayFactory,
                { :type => XSD::XSDString::Type })
        @drv.mapping_registry = @map
      end

      def get_bugs(key, value, key2, value2)
        ### soap interface to get bugs with matching key/value.
        @drv.get_bugs(key, value, key2, value2)
      end

      def get_status(sa)
        ### soap interface to get bug status matching the bug number(s). sa is a array of strings.
        get_status = Debian::Bugs.new
        if sa.size == 0
          puts "No bugs to fetch" if $DEBUG
        else
          puts "fetching #{sa.join(" ")}.. " if $DEBUG
          @drv.get_status(sa).each { |bugnum, res|
            newbug=Debian::Bug.new(res.package, 
                                   bugnum.to_s, 
                                   res.severity,
                                   res.pending, 
                                   res.subject,
                                   res.tags.split(" "),
                                   res.mergedwith.to_s.split(" "),
                                   Time::at(res.date.to_i))
            p res if $DEBUG
            newbug.found=res.found.keys.join(" ") if res.found.kind_of?(Hash)
            newbug.fixed=res.fixed.keys.join(" ") if res.fixed.kind_of?(Hash)
            get_status << newbug
          }
        end
        get_status
      end
    end
  end
end
